@@ -1,11 +1,10 @@
-from zeno.api import distill, metric, model, transform, ZenoOptions
+from zeno.api import distill, metric, model, ZenoOptions
 from zeno.runner import zeno
 
 __all__ = [
     "model",
     "distill",
     "metric",
-    "transform",
     "ZenoOptions",
     "zeno",
 ]
