@@ -6,20 +6,18 @@ from pathlib import Path
 import nest_asyncio  # type: ignore
 import pandas as pd
 import pkg_resources
-import requests  # type: ignore
+import requests
 import tomli
 import uvicorn
 
 from zeno.server import get_server
-from zeno.util import parse_testing_file  # type: ignore
+from zeno.setup import setup_zeno
+from zeno.util import is_notebook, parse_testing_file, VIEW_MAP_URL, VIEWS_MAP_JSON
 from zeno.zeno_backend import ZenoBackend
 
-VIEW_MAP_URL = "https://raw.githubusercontent.com/zeno-ml/instance-views/main/"
-VIEWS_MAP_JSON = "views.json"
-
 
 def command_line():
-    if len(sys.argv) == 1:
+    if len(sys.argv) == 1 or sys.argv[1] == "-h" or sys.argv[1] == "--help":
         print(
             "\n \033[1mZeno\033[0m",
             pkg_resources.get_distribution("zenoml").version,
@@ -39,7 +37,8 @@ def command_line():
 
     if len(sys.argv) != 2:
         print(
-            "ERROR: Zeno take one argument, a configuration TOML file. "
+            "ERROR: Zeno take one argument, either a configuration TOML file"
+            + " or the keyword 'init'. "
             + "{0} arguments were passed.",
             len(sys.argv),
         )
@@ -49,6 +48,13 @@ def command_line():
         print(pkg_resources.get_distribution("zenoml").version)
         sys.exit(0)
 
+    if sys.argv[1] == "init" or sys.argv[1] == "i":
+        setup_zeno()
+    else:
+        parse_toml()
+
+
+def parse_toml():
     args = {}
     try:
         with open(sys.argv[1], "rb") as f:
@@ -77,7 +83,6 @@ def command_line():
     if "functions" not in args or not os.path.exists(
         os.path.realpath(os.path.join(base_path, args["functions"]))
     ):
-        print("WARNING: No 'functions' directory found.")
         args["functions"] = []
     else:
         args["functions"] = Path(
@@ -130,7 +135,6 @@ def parse_args(args: dict, base_path) -> dict:
             sys.exit(1)
 
     if "models" not in args or len(args["models"]) < 1:
-        print("WARNING: No 'models' found.")
         args["models"] = []
     else:
         if Path(os.path.realpath(os.path.join(base_path, args["models"][0]))).exists():
@@ -139,6 +143,9 @@ def parse_args(args: dict, base_path) -> dict:
                 for m in args["models"]
             ]
 
+    if "functions" not in args:
+        args["functions"] = []
+
     if "data_path" not in args:
         args["data_path"] = ""
     elif not args["data_path"].startswith("http"):
@@ -193,7 +200,9 @@ def parse_args(args: dict, base_path) -> dict:
 
 def zeno(args, base_path="./"):
     nest_asyncio.apply()
+
     args = parse_args(args, base_path)
+
     zeno = ZenoBackend(
         df=args["metadata"],
         functions=args["functions"],
@@ -214,6 +223,22 @@ def zeno(args, base_path="./"):
     app = get_server(zeno)
 
     if args["serve"]:
-        uvicorn.run(app, host=args["host"], port=args["port"])
+        print(
+            "\n\033[1mZeno\033[0m running on http://{}:{}\n".format(
+                args["host"], args["port"]
+            )
+        )
+        if is_notebook():
+            from IPython.display import IFrame  # type: ignore
+
+            display(  # noqa: F821 # type: ignore
+                IFrame(
+                    "http://{}:{}".format(args["host"], args["port"]),
+                    "100%",
+                    800,
+                )
+            )
+
+        uvicorn.run(app, host=args["host"], port=args["port"], log_level="critical")
     else:
         return app
