@@ -13,6 +13,9 @@ import pandas as pd  # type: ignore
 
 from zeno.classes import MetadataType
 
+VIEW_MAP_URL = "https://raw.githubusercontent.com/zeno-ml/instance-views/main/"
+VIEWS_MAP_JSON = "views.json"
+
 
 class NpEncoder(json.JSONEncoder):
     def default(self, obj):
@@ -78,6 +81,8 @@ def load_series(df, col_name, save_path):
         df.loc[:, col_name] = pd.read_pickle(save_path)
     except FileNotFoundError:
         df.loc[:, col_name] = pd.Series([pd.NA] * df.shape[0], index=df.index)
+    except EOFError:
+        df.loc[:, col_name] = pd.Series([pd.NA] * df.shape[0], index=df.index)
 
 
 @contextmanager
@@ -105,8 +110,20 @@ def parse_testing_file(test_file: Path):
             if (
                 hasattr(func, "predict_function")
                 or hasattr(func, "distill_function")
-                or hasattr(func, "transform_function")
                 or hasattr(func, "metric_function")
             ):
                 functions.append(func)
     return functions
+
+
+def is_notebook() -> bool:
+    try:
+        shell = get_ipython().__class__.__name__  # type: ignore
+        if shell == "ZMQInteractiveShell":
+            return True  # Jupyter notebook or qtconsole
+        elif shell == "TerminalInteractiveShell":
+            return False  # Terminal running IPython
+        else:
+            return False  # Other type (?)
+    except NameError:
+        return False  # Probably standard Python interpreter
