@@ -5,6 +5,7 @@ import os
 import pickle
 import sys
 import threading
+from functools import lru_cache
 from inspect import getsource
 from math import isnan
 from pathlib import Path
@@ -28,14 +29,8 @@ from zeno.classes import (
     ZenoColumnType,
     ZenoState,
 )
-from zeno.data_processing import (
-    postdistill_data,
-    predistill_data,
-    run_inference,
-    transform_data,
-)
+from zeno.data_processing import postdistill_data, predistill_data, run_inference
 from zeno.filtering import filter_table, filter_table_single
-from zeno.mirror import Mirror
 from zeno.util import getMetadataType, load_series, read_pickle
 
 
@@ -72,7 +67,6 @@ class ZenoBackend(object):
         self.predistill_functions: Dict[str, Callable] = {}
         self.postdistill_functions: Dict[str, Callable] = {}
         self.metric_functions: Dict[str, Callable] = {}
-        self.transform_functions: Dict[str, Callable] = {}
         self.predict_function: Optional[Callable] = None
 
         self.status: str = "Initializing"
@@ -92,7 +86,6 @@ class ZenoBackend(object):
 
         self.__setup_dataframe(id_column, data_column, label_column)
         self.__parse_test_functions(self.tests)
-        self.mirror = Mirror(self.df, self.cache_path, self.id_column)
 
         # Options passed to Zeno functions.
         self.zeno_options = ZenoOptions(
@@ -134,7 +127,7 @@ class ZenoBackend(object):
             col = ZenoColumn(
                 column_type=ZenoColumnType.METADATA,
                 metadata_type=getMetadataType(self.df[metadata_col]),
-                name=metadata_col[1:],
+                name=str(metadata_col)[1:],
             )
             self.columns.append(col)
             self.complete_columns.append(col)
@@ -153,12 +146,8 @@ class ZenoBackend(object):
                     self.postdistill_functions[test_fn.__name__] = test_fn
                 else:
                     self.predistill_functions[test_fn.__name__] = test_fn
-            if hasattr(test_fn, "transform_function"):
-                self.transform_functions[test_fn.__name__] = test_fn
             if hasattr(test_fn, "metric_function"):
                 self.metric_functions[test_fn.__name__] = test_fn
-        if not self.predict_function:
-            print("WARNING: No model function found")
 
     def start_processing(self):
         """Parse testing files, distill, and run inference."""
@@ -186,49 +175,25 @@ class ZenoBackend(object):
         self.__thread.start()
 
     async def __process(self):
-        self.status = "Running distill functions"
+        self.status = "Running predistill functions"
+        print(self.status)
         self.__predistill()
-        self.status = "Running transform functions"
-        self.__transform()
 
-        for transform in [*self.transform_functions.values(), None]:
-            self.__inference_and_postdistill(transform)
+        self.status = "Running inference"
+        print(self.status)
+        self.__inference()
 
+        self.status = "Running postdistill functions"
+        print(self.status)
+        self.__postdistill()
+
+        self.status = "Done processing"
+        print(self.status)
         self.done_processing = True
 
-    def __transform(self):
-        """Run transform functions."""
-        transform_to_run: List[Callable] = []
-        for transform_function in self.transform_functions.values():
-            transform_column = ZenoColumn(
-                column_type=ZenoColumnType.TRANSFORM, name=transform_function.__name__
-            )
-            save_path = Path(self.cache_path, str(transform_column) + ".pickle")
-
-            load_series(self.df, str(transform_column), save_path)
-
-            if self.df[str(transform_column)].isnull().any():
-                transform_to_run.append(transform_function)
-            else:
-                self.complete_columns.append(transform_column)
-
-        if len(transform_to_run) > 0:
-            with Pool() as pool:
-                transform_outputs = pool.map(
-                    transform_data,
-                    [fn for fn in transform_to_run],
-                    [self.zeno_options] * len(transform_to_run),
-                    [self.cache_path] * len(transform_to_run),
-                    [self.df] * len(transform_to_run),
-                    [self.batch_size] * len(transform_to_run),
-                    range(len(transform_to_run)),
-                )
-                for out in transform_outputs:
-                    self.df.loc[:, str(out[0])] = out[1]
-                    self.complete_columns.append(out[0])
-
     def __predistill(self):
         """Run distilling functions not dependent on model outputs."""
+
         # Check if we need to preprocess since Pool is expensive
         predistill_to_run: List[ZenoColumn] = []
         for predistill_column in [
@@ -263,7 +228,9 @@ class ZenoBackend(object):
                     out[0].metadata_type = getMetadataType(self.df[str(out[0])])
                     self.complete_columns.append(out[0])
 
-    def __inference_and_postdistill(self, transform: Optional[Callable]):
+    def __inference(self):
+        """Run models on instances."""
+
         # Check if we need to run inference since Pool is expensive
         models_to_run = []
         for model_path in self.model_paths:
@@ -271,12 +238,10 @@ class ZenoBackend(object):
             model_column = ZenoColumn(
                 column_type=ZenoColumnType.OUTPUT,
                 name=model_name,
-                transform=transform.__name__ if transform else "",
             )
             embedding_column = ZenoColumn(
                 column_type=ZenoColumnType.EMBEDDING,
                 name=model_name,
-                transform=transform.__name__ if transform else "",
             )
             model_hash = str(model_column)
             embedding_hash = str(embedding_column)
@@ -295,14 +260,12 @@ class ZenoBackend(object):
             else:
                 self.complete_columns.append(model_column)
 
-        self.status = "Running inference"
         if len(models_to_run) > 0:
             with Pool() as pool:
                 inference_outputs = pool.map(
                     run_inference,
                     [self.predict_function] * len(models_to_run),
                     [self.zeno_options] * len(models_to_run),
-                    [transform.__name__ if transform else ""] * len(models_to_run),
                     [m for m in models_to_run],
                     [self.cache_path] * len(models_to_run),
                     [self.df] * len(models_to_run),
@@ -315,7 +278,8 @@ class ZenoBackend(object):
                         self.df.loc[:, str(out[1])] = out[3]
                     self.complete_columns.append(out[0])
 
-        self.status = "Done running inference"
+    def __postdistill(self):
+        """Run distill functions dependent on model outputs."""
 
         # Check if we need to run postprocessing since Pool is expensive
         postdistill_to_run: List[ZenoColumn] = []
@@ -325,7 +289,6 @@ class ZenoBackend(object):
             col_name = postdistill_column.copy(
                 update={
                     "model": postdistill_column.model,
-                    "transform": transform.__name__ if transform else "",
                 }
             )
             col_hash = str(col_name)
@@ -339,14 +302,12 @@ class ZenoBackend(object):
                 col_name.metadata_type = getMetadataType(self.df[col_hash])
                 self.complete_columns.append(col_name)
 
-        self.status = "Running postprocessing"
         if len(postdistill_to_run) > 0:
             with Pool() as pool:
                 post_outputs = pool.map(
                     postdistill_data,
                     [self.postdistill_functions[e.name] for e in postdistill_to_run],
                     [e.model for e in postdistill_to_run],
-                    [transform.__name__ if transform else ""] * len(postdistill_to_run),
                     [self.zeno_options] * len(postdistill_to_run),
                     [self.cache_path] * len(postdistill_to_run),
                     [self.df] * len(postdistill_to_run),
@@ -358,8 +319,6 @@ class ZenoBackend(object):
                     out[0].metadata_type = getMetadataType(self.df[str(out[0])])
                     self.complete_columns.append(out[0])
 
-        self.status = "Done running postprocessing"
-
     def get_metrics_for_slices(self, requests: List[MetricKey]):
         """Calculate result for each requested combination."""
 
@@ -380,7 +339,6 @@ class ZenoBackend(object):
         output_col = ZenoColumn(
             column_type=ZenoColumnType.OUTPUT,
             name=state.model,
-            transform=state.transform,
         )
         output_hash = str(output_col)
 
@@ -391,7 +349,6 @@ class ZenoBackend(object):
                 c.column_type == ZenoColumnType.PREDISTILL
                 or c.column_type == ZenoColumnType.POSTDISTILL
             )
-            and c.transform == state.transform
             and c.model == state.model
         ]
 
@@ -469,6 +426,15 @@ class ZenoBackend(object):
             req.get_metrics = False
 
         cols = req.columns
+
+        # For each histogram request return a list of buckets like:
+        # {
+        #    bucket: start value
+        #    bucketEnd: end value (optional)
+        #    count: number of entries in this bucket
+        #    filteredCount: number of entries in this bucket after filtering
+        #    metric: metric value for this bucket (optional)
+        # }
         res = {}
         for col in cols:
             df_col = self.df[str(col)]
@@ -500,8 +466,8 @@ class ZenoBackend(object):
                 for i, bin_count in enumerate(bins[0]):
                     ret_hist.append(
                         {
-                            "bucket": round(bins[1][i], 2),
-                            "bucketEnd": round(bins[1][i + 1], 2),
+                            "bucket": round(bins[1][i], 4),
+                            "bucketEnd": round(bins[1][i + 1], 4),
                             "count": int(bin_count),
                             "filteredCount": int(bins_filt[0][i]),
                             "metric": self.get_histogram_metric(
@@ -531,6 +497,45 @@ class ZenoBackend(object):
                 res[str(col)] = []
         return res
 
-    def embedding_exists(self, model: str):
-        col = ZenoColumn(name=model, column_type=ZenoColumnType.EMBEDDING)
-        return str(col) in self.df.columns
+    def embed_exists(self, model: str):
+        """Checks for the existence of an embedding column.
+        Returns True if the column exists, False otherwise
+        """
+        embed_column = ZenoColumn(name=model, column_type=ZenoColumnType.EMBEDDING)
+        exists = str(embed_column) in self.df.columns
+        return exists and not self.df[str(embed_column)].isnull().any()
+
+    @lru_cache()
+    def project_embed_into_2D(self, model: str) -> Dict[str, list]:
+        """If the embedding exists, will use t-SNE to project into 2D.
+        Returns the 2D embeddings as object/dict
+        {
+            x: list[float]
+            y: list[float]
+            ids: list[str]
+        }
+        """
+
+        points: Dict[str, list] = {"x": [], "y": [], "ids": []}
+
+        # Can't project without an embedding
+        if not self.embed_exists(model):
+            return points
+
+        embed_col = ZenoColumn(column_type=ZenoColumnType.EMBEDDING, name=model)
+
+        # Extract embeddings and store in one big ndarray
+        embed = self.df[str(embed_col)].to_numpy()
+        embed = np.stack(embed, axis=0)  # type: ignore
+
+        # project embeddings into 2D
+        from sklearn.manifold import TSNE  # type: ignore
+
+        projection = TSNE().fit_transform(embed)
+
+        # extract points and ids from computed projection
+        points["x"] = projection[:, 0].tolist()
+        points["y"] = projection[:, 1].tolist()
+        points["ids"] = self.df[str(self.id_column)].to_list()
+
+        return points
